@@ -4,7 +4,7 @@
 #include<yade/pkg/dem/Shop.hpp>
 #include<yade/lib/smoothing/LinearInterpolate.hpp>
 
-YADE_PLUGIN((KinematicEngine)(CombinedKinematicEngine)(TranslationEngine)(HarmonicMotionEngine)(RotationEngine)(HelixEngine)(InterpolatingHelixEngine)(HarmonicRotationEngine));
+YADE_PLUGIN((KinematicEngine)(CombinedKinematicEngine)(TranslationEngine)(HarmonicMotionEngine)(RotationEngine)(HelixEngine)(InterpolatingHelixEngine)(HarmonicRotationEngine)(ServoPIDController));
 
 CREATE_LOGGER(KinematicEngine);
 
@@ -31,6 +31,7 @@ void CombinedKinematicEngine::action(){
 		}
 		// apply one engine after another
 		FOREACH(const shared_ptr<KinematicEngine>& e, comb){
+			if (e->dead) continue;
 			e->scene=scene; e->apply(ids);
 		}
 	} else {
@@ -142,3 +143,43 @@ void HarmonicRotationEngine::apply(const vector<Body::id_t>& ids){
 	RotationEngine::apply(ids);
 }
 
+void ServoPIDController::apply(const vector<Body::id_t>& ids){
+  
+  if (iterPrevStart < 0 or ((scene->iter-iterPrevStart)>=iterPeriod)) {
+  
+    Vector3r tmpForce = Vector3r::Zero();
+    
+    if (ids.size()>0) {
+      FOREACH(Body::id_t id,ids){
+        assert(id<(Body::id_t)scene->bodies->size());
+        tmpForce += scene->forces.getForce(id);
+      }
+    } else {
+      LOG_WARN("The list of ids is empty!");
+    }
+    
+    axis.normalize();
+    tmpForce = tmpForce.cwiseProduct(axis);   // Take into account given axis
+    errorCur = tmpForce.norm() - target;      // Find error
+    
+    const Real pTerm = errorCur*kP;               // Calculate proportional term
+    iTerm += errorCur*kI;                         // Calculate integral term
+    const Real dTerm = (errorCur-errorPrev)*kD;   // Calculate derivative term
+    
+    errorPrev = errorCur;                         // Save the current value of the error
+    
+    curVel = (pTerm + iTerm + dTerm);             // Calculate current velocity
+    
+    if (fabs(curVel) > fabs(maxVelocity)) {
+      curVel*=fabs(maxVelocity)/fabs(curVel);
+    }
+    
+    iterPrevStart = scene->iter;
+  }
+  
+  translationAxis = axis;
+  velocity = curVel;
+  
+  TranslationEngine::apply(ids);
+}
+
