@@ -16,6 +16,7 @@
 #include<vtkTriangle.h>
 #include<vtkLine.h>
 #include<vtkQuad.h>
+#include<vtkHexahedron.h>
 #ifdef YADE_VTK_MULTIBLOCK
   #include<vtkXMLMultiBlockDataWriter.h>
   #include<vtkMultiBlockDataSet.h>
@@ -26,8 +27,8 @@
 #include<yade/pkg/common/Facet.hpp>
 #include<yade/pkg/common/Box.hpp>
 #include<yade/pkg/dem/ConcretePM.hpp>
-#include<yade/pkg/dem/RockPM.hpp>
 #include<yade/pkg/dem/WirePM.hpp>
+#include<yade/pkg/dem/JointedCohesiveFrictionalPM.hpp>
 #include<yade/pkg/dem/Shop.hpp>
 
 
@@ -50,6 +51,7 @@ void VTKRecorder::action(){
 			recActive[REC_CLUMPID]=true;
 			recActive[REC_MATERIALID]=true;
 			recActive[REC_STRESS]=true;
+			if (scene->isPeriodic) { recActive[REC_PERICELL]=true; }
 		}
 		else if(rec=="spheres") recActive[REC_SPHERES]=true;
 		else if(rec=="velocity") recActive[REC_VELOCITY]=true;
@@ -58,7 +60,6 @@ void VTKRecorder::action(){
 		else if(rec=="mass") recActive[REC_MASS]=true;
 		else if((rec=="colors") || (rec=="color"))recActive[REC_COLORS]=true;
 		else if(rec=="cpm") recActive[REC_CPM]=true;
-		else if(rec=="rpm") recActive[REC_RPM]=true;
 		else if(rec=="wpm") recActive[REC_WPM]=true;
 		else if(rec=="intr") recActive[REC_INTR]=true;
 		else if((rec=="ids") || (rec=="id")) recActive[REC_ID]=true;
@@ -66,11 +67,17 @@ void VTKRecorder::action(){
 		else if((rec=="clumpids") || (rec=="clumpId")) recActive[REC_CLUMPID]=true;
 		else if(rec=="materialId") recActive[REC_MATERIALID]=true;
 		else if(rec=="stress") recActive[REC_STRESS]=true;
-		else LOG_ERROR("Unknown recorder named `"<<rec<<"' (supported are: all, spheres, velocity, facets, boxes, color, stress, cpm, rpm, wpm, intr, id, clumpId, materialId). Ignored.");
+		else if(rec=="jcfpm") recActive[REC_JCFPM]=true;
+		else if(rec=="cracks") recActive[REC_CRACKS]=true;
+		else if(rec=="pericell" && scene->isPeriodic) recActive[REC_PERICELL]=true;
+		else LOG_ERROR("Unknown recorder named `"<<rec<<"' (supported are: all, spheres, velocity, facets, boxes, color, stress, cpm, wpm, intr, id, clumpId, materialId, jcfpm, cracks, pericell). Ignored.");
 	}
 	// cpm needs interactions
 	if(recActive[REC_CPM]) recActive[REC_INTR]=true;
 	
+	// jcfpm needs interactions
+	if(recActive[REC_JCFPM]) recActive[REC_INTR]=true;
+
 	// wpm needs interactions
 	if(recActive[REC_WPM]) recActive[REC_INTR]=true;
 	
@@ -191,6 +198,10 @@ void VTKRecorder::action(){
 	intrAbsForceT->SetNumberOfComponents(3);
 	intrAbsForceT->SetName("absForceT");
 
+	// pericell
+	vtkSmartPointer<vtkPoints> pericellPoints = vtkSmartPointer<vtkPoints>::New();
+	vtkSmartPointer<vtkCellArray> pericellHexa = vtkSmartPointer<vtkCellArray>::New();
+
 	// extras for CPM
 	if(recActive[REC_CPM]){ CpmStateUpdater csu; csu.update(scene); }
 	vtkSmartPointer<vtkFloatArray> cpmDamage = vtkSmartPointer<vtkFloatArray>::New();
@@ -200,17 +211,52 @@ void VTKRecorder::action(){
 	cpmStress->SetNumberOfComponents(9);
 	cpmStress->SetName("cpmStress");
 
-	// extras for RPM
-	vtkSmartPointer<vtkFloatArray> rpmSpecNum = vtkSmartPointer<vtkFloatArray>::New();
-	rpmSpecNum->SetNumberOfComponents(1);
-	rpmSpecNum->SetName("rpmSpecNum");
-	vtkSmartPointer<vtkFloatArray> rpmSpecMass = vtkSmartPointer<vtkFloatArray>::New();
-	rpmSpecMass->SetNumberOfComponents(1);
-	rpmSpecMass->SetName("rpmSpecMass");
-	vtkSmartPointer<vtkFloatArray> rpmSpecDiam = vtkSmartPointer<vtkFloatArray>::New();
-	rpmSpecDiam->SetNumberOfComponents(1);
-	rpmSpecDiam->SetName("rpmSpecDiam");
+	// extras for JCFpm
+	vtkSmartPointer<vtkFloatArray> damage = vtkSmartPointer<vtkFloatArray>::New();
+	damage->SetNumberOfComponents(1);;
+	damage->SetName("damage");
+	vtkSmartPointer<vtkFloatArray> damageRel = vtkSmartPointer<vtkFloatArray>::New();
+	damageRel->SetNumberOfComponents(1);;
+	damageRel->SetName("damageRel");
+	vtkSmartPointer<vtkFloatArray> intrIsCohesive = vtkSmartPointer<vtkFloatArray>::New();
+	intrIsCohesive->SetNumberOfComponents(1);
+	intrIsCohesive->SetName("isCohesive");
+	vtkSmartPointer<vtkFloatArray> intrIsOnJoint = vtkSmartPointer<vtkFloatArray>::New();
+	intrIsOnJoint->SetNumberOfComponents(1);
+	intrIsOnJoint->SetName("isOnJoint");
+	
+	// extras for cracks
+	vtkSmartPointer<vtkPoints> crackPos = vtkSmartPointer<vtkPoints>::New();
+	vtkSmartPointer<vtkCellArray> crackCells = vtkSmartPointer<vtkCellArray>::New();
+	vtkSmartPointer<vtkFloatArray> iter = vtkSmartPointer<vtkFloatArray>::New();
+	iter->SetNumberOfComponents(1);
+	iter->SetName("iter");
+	vtkSmartPointer<vtkFloatArray> crackType = vtkSmartPointer<vtkFloatArray>::New();
+	crackType->SetNumberOfComponents(1);
+	crackType->SetName("crackType");
+	vtkSmartPointer<vtkFloatArray> crackSize = vtkSmartPointer<vtkFloatArray>::New();
+	crackSize->SetNumberOfComponents(1);
+	crackSize->SetName("crackSize");
+	vtkSmartPointer<vtkFloatArray> crackNorm = vtkSmartPointer<vtkFloatArray>::New();
+	crackNorm->SetNumberOfComponents(3);
+	crackNorm->SetName("crackNorm");
 
+// 	// the same for newly created cracks
+// 	vtkSmartPointer<vtkPoints> crackPosNew = vtkSmartPointer<vtkPoints>::New();
+// 	vtkSmartPointer<vtkCellArray> crackCellsNew = vtkSmartPointer<vtkCellArray>::New();
+// 	vtkSmartPointer<vtkFloatArray> iterNew = vtkSmartPointer<vtkFloatArray>::New();
+// 	iterNew->SetNumberOfComponents(1);
+// 	iterNew->SetName("iter");
+// 	vtkSmartPointer<vtkFloatArray> crackTypeNew = vtkSmartPointer<vtkFloatArray>::New();
+// 	crackTypeNew->SetNumberOfComponents(1);
+// 	crackTypeNew->SetName("crackType");
+// 	vtkSmartPointer<vtkFloatArray> crackSizeNew = vtkSmartPointer<vtkFloatArray>::New();
+// 	crackSizeNew->SetNumberOfComponents(1);
+// 	crackSizeNew->SetName("crackSize");
+// 	vtkSmartPointer<vtkFloatArray> crackNormNew = vtkSmartPointer<vtkFloatArray>::New();
+// 	crackNormNew->SetNumberOfComponents(3);
+// 	crackNormNew->SetName("crackNorm");
+	
 	// extras for WireMatPM
 	vtkSmartPointer<vtkFloatArray> wpmNormalForce = vtkSmartPointer<vtkFloatArray>::New();
 	wpmNormalForce->SetNumberOfComponents(1);
@@ -296,6 +342,13 @@ void VTKRecorder::action(){
 						wpmLimitFactor->InsertNextValue(NaN);
 					}
 				}
+				else if (recActive[REC_JCFPM]){
+					const JCFpmPhys* jcfpmphys = YADE_CAST<JCFpmPhys*>(I->phys.get());
+					intrIsCohesive->InsertNextValue(jcfpmphys->isCohesive);
+					intrIsOnJoint->InsertNextValue(jcfpmphys->isOnJoint);
+					intrForceN->InsertNextValue(fn);
+				}
+
 				else {
 					intrForceN->InsertNextValue(fn);
 				}
@@ -356,12 +409,12 @@ void VTKRecorder::action(){
 					float s[9]={ (float) ss(0,0), (float) ss(0,1), (float) ss(0,2), (float) ss(1,0), (float) ss(1,1), (float) ss(1,2), (float) ss(2,0), (float) ss(2,1), (float) ss(2,2)};
 					cpmStress->InsertNextTupleValue(s);
 				}
-				if (recActive[REC_RPM]){
-					rpmSpecNum->InsertNextValue(YADE_PTR_CAST<RpmState>(b->state)->specimenNumber);
-					rpmSpecMass->InsertNextValue(YADE_PTR_CAST<RpmState>(b->state)->specimenMass);
-					rpmSpecDiam->InsertNextValue(YADE_PTR_CAST<RpmState>(b->state)->specimenMaxDiam);
-				}
 				
+				if (recActive[REC_JCFPM]){
+					damage->InsertNextValue(YADE_PTR_CAST<JCFpmState>(b->state)->tensBreak + YADE_PTR_CAST<JCFpmState>(b->state)->shearBreak);
+// 					damageRel->InsertNextValue(YADE_PTR_CAST<JCFpmState>(b->state)->tensBreakRel + YADE_PTR_CAST<JCFpmState>(b->state)->shearBreakRel);
+				}
+
 				if (recActive[REC_MATERIALID]) spheresMaterialId->InsertNextValue(b->material->id);
 				continue;
 			}
@@ -451,6 +504,32 @@ void VTKRecorder::action(){
 		}
 	}
 
+	if (recActive[REC_PERICELL]) {
+		const Matrix3r& hSize = scene->cell->hSize;
+		Vector3r v0 = hSize*Vector3r(0,0,1);
+		Vector3r v1 = hSize*Vector3r(0,1,1);
+		Vector3r v2 = hSize*Vector3r(1,1,1);
+		Vector3r v3 = hSize*Vector3r(1,0,1);
+		Vector3r v4 = hSize*Vector3r(0,0,0);
+		Vector3r v5 = hSize*Vector3r(0,1,0);
+		Vector3r v6 = hSize*Vector3r(1,1,0);
+		Vector3r v7 = hSize*Vector3r(1,0,0);
+		pericellPoints->InsertNextPoint(v0[0],v0[1],v0[2]);
+		pericellPoints->InsertNextPoint(v1[0],v1[1],v1[2]);
+		pericellPoints->InsertNextPoint(v2[0],v2[1],v2[2]);
+		pericellPoints->InsertNextPoint(v3[0],v3[1],v3[2]);
+		pericellPoints->InsertNextPoint(v4[0],v4[1],v4[2]);
+		pericellPoints->InsertNextPoint(v5[0],v5[1],v5[2]);
+		pericellPoints->InsertNextPoint(v6[0],v6[1],v6[2]);
+		pericellPoints->InsertNextPoint(v7[0],v7[1],v7[2]);
+		vtkSmartPointer<vtkHexahedron> h = vtkSmartPointer<vtkHexahedron>::New();
+		vtkIdList* l = h->GetPointIds();
+		for (int i=0; i<8; i++) {
+			l->SetId(i,i);
+		}
+		pericellHexa->InsertNextCell(h);
+	}
+
 	
 	vtkSmartPointer<vtkDataCompressor> compressor;
 	if(compress) compressor=vtkSmartPointer<vtkZLibDataCompressor>::New();
@@ -480,14 +559,12 @@ void VTKRecorder::action(){
 			spheresUg->GetPointData()->AddArray(cpmDamage);
 			spheresUg->GetPointData()->AddArray(cpmStress);
 		}
-		if (recActive[REC_RPM]){
-			spheresUg->GetPointData()->AddArray(rpmSpecNum);
-			spheresUg->GetPointData()->AddArray(rpmSpecMass);
-			spheresUg->GetPointData()->AddArray(rpmSpecDiam);
-		}
 
+		if (recActive[REC_JCFPM]) {
+			spheresUg->GetPointData()->AddArray(damage);
+		}
 		if (recActive[REC_MATERIALID]) spheresUg->GetPointData()->AddArray(spheresMaterialId);
-		
+
 		#ifdef YADE_VTK_MULTIBLOCK
 		if(!multiblock)
 		#endif
@@ -555,6 +632,10 @@ void VTKRecorder::action(){
 		intrPd->SetLines(intrCells);
 		intrPd->GetCellData()->AddArray(intrForceN);
 		intrPd->GetCellData()->AddArray(intrAbsForceT);
+		if (recActive[REC_JCFPM]) { 
+			intrPd->GetCellData()->AddArray(intrIsCohesive);
+			intrPd->GetCellData()->AddArray(intrIsOnJoint);
+		}
 		if (recActive[REC_WPM]){
 			intrPd->GetCellData()->AddArray(wpmNormalForce);
 			intrPd->GetCellData()->AddArray(wpmLimitFactor);
@@ -572,6 +653,87 @@ void VTKRecorder::action(){
 			writer->Write();
 		}
 	}
+	vtkSmartPointer<vtkUnstructuredGrid> pericellUg = vtkSmartPointer<vtkUnstructuredGrid>::New();
+	if (recActive[REC_PERICELL]){
+		pericellUg->SetPoints(pericellPoints);
+		pericellUg->SetCells(12,pericellHexa);
+		#ifdef YADE_VTK_MULTIBLOCK
+			if(!multiblock)
+		#endif
+			{
+			vtkSmartPointer<vtkXMLUnstructuredGridWriter> writer = vtkSmartPointer<vtkXMLUnstructuredGridWriter>::New();
+			if(compress) writer->SetCompressor(compressor);
+			if(ascii) writer->SetDataModeToAscii();
+			string fn=fileName+"pericell."+lexical_cast<string>(scene->iter)+".vtu";
+			writer->SetFileName(fn.c_str());
+			writer->SetInput(pericellUg);
+			writer->Write();
+		}
+	}
+
+	if (recActive[REC_CRACKS]) {
+		std::ifstream file ("cracks.txt",std::ios::in);
+		vtkSmartPointer<vtkUnstructuredGrid> crackUg = vtkSmartPointer<vtkUnstructuredGrid>::New();
+		vtkSmartPointer<vtkUnstructuredGrid> crackUgNew = vtkSmartPointer<vtkUnstructuredGrid>::New();
+		
+		 if(file){
+			 while ( !file.eof() ){
+				std::string line;
+				Real i,p0,p1,p2,t,s,n0,n1,n2;
+				while ( std::getline(file, line)/* writes into string "line", a line of file "file". To go along diff. lines*/ ) 
+				{
+					file >> i >> p0 >> p1 >> p2 >> t >> s >> n0 >> n1 >> n2;
+					vtkIdType pid[1];
+					pid[0] = crackPos->InsertNextPoint(p0, p1, p2);
+					crackCells->InsertNextCell(1,pid);
+					crackType->InsertNextValue(t);
+					crackSize->InsertNextValue(s);
+					iter->InsertNextValue(i);
+					float n[3] = { n0, n1, n2 };
+					crackNorm->InsertNextTupleValue(n);
+// 					if (i > scene->iter - iterPeriod)
+// 					{
+// 					  pid[0] = crackPosNew->InsertNextPoint(p0, p1, p2);
+// 					  crackCellsNew->InsertNextCell(1,pid);
+// 					  crackTypeNew->InsertNextValue(t);
+// 					  crackSizeNew->InsertNextValue(s);
+// 					  iterNew->InsertNextValue(i);
+// 					  crackNormNew->InsertNextTupleValue(n);
+// 					}
+					  
+				}
+			 }
+			 file.close();
+		} 
+
+		crackUg->SetPoints(crackPos);
+		crackUg->SetCells(VTK_VERTEX, crackCells);
+		crackUg->GetPointData()->AddArray(iter);
+		crackUg->GetPointData()->AddArray(crackType);
+		crackUg->GetPointData()->AddArray(crackSize);
+		crackUg->GetPointData()->AddArray(crackNorm); //orientation of 2D glyphs does not match this direction (some work to do in order to have the good orientation) 
+		
+		/*crackUgNew->SetPoints(crackPosNew);
+		crackUgNew->SetCells(VTK_VERTEX, crackCellsNew);
+		crackUgNew->GetPointData()->AddArray(iterNew);
+		crackUgNew->GetPointData()->AddArray(crackTypeNew);
+		crackUgNew->GetPointData()->AddArray(crackSizeNew);
+		crackUgNew->GetPointData()->AddArray(crackNormNew); //same remark...*/
+	
+		vtkSmartPointer<vtkXMLUnstructuredGridWriter> writer = vtkSmartPointer<vtkXMLUnstructuredGridWriter>::New();
+		if(compress) writer->SetCompressor(compressor);
+		if(ascii) writer->SetDataModeToAscii();
+		string fn=fileName+"cracks."+lexical_cast<string>(scene->iter)+".vtu";
+		writer->SetFileName(fn.c_str());
+		writer->SetInput(crackUg);
+		writer->Write();
+		
+// 		fn=fileName+"newcracks."+lexical_cast<string>(scene->iter)+".vtu";
+// 		writer->SetFileName(fn.c_str());
+// 		writer->SetInput(crackUgNew);
+// 		writer->Write();
+	}
+
 	#ifdef YADE_VTK_MULTIBLOCK
 		if(multiblock){
 			vtkSmartPointer<vtkMultiBlockDataSet> multiblockDataset = vtkSmartPointer<vtkMultiBlockDataSet>::New();
@@ -579,6 +741,7 @@ void VTKRecorder::action(){
 			if(recActive[REC_SPHERES]) multiblockDataset->SetBlock(i++,spheresUg);
 			if(recActive[REC_FACETS]) multiblockDataset->SetBlock(i++,facetsUg);
 			if(recActive[REC_INTR]) multiblockDataset->SetBlock(i++,intrPd);
+			if(recActive[REC_PERICELL]) multiblockDataset->SetBlock(i++,pericellUg);
 			vtkSmartPointer<vtkXMLMultiBlockDataWriter> writer = vtkSmartPointer<vtkXMLMultiBlockDataWriter>::New();
 			if(ascii) writer->SetDataModeToAscii();
 			string fn=fileName+lexical_cast<string>(scene->iter)+".vtm";
